<?php

  /*

   Generic PHP Netflix Parsing Class

   * Copyright *
   Parts Copyright 2004, by Paul Burdick (reedmaniac@pmachine.com)
   Parts Copyright 2004, by James McDuffie (ttq3sxl02@sneakemail.com)

   * License *
   GNU GPL

   * Description *

   Downloads a user's Netflix queue or out list from the Netflix website
   then caches and parses it.

   * Example Usage *

    $netflix_id = 'PXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    $netflix_cache_dir = '/tmp/netflix_cache';

    $nf_parse = new NetFlixParse($netflix_id, $netflix_cache_dir);
    $nf_parse->parse_data($which_list);
    $items = $nf_parse->get_items();

    foreach ($items as $curr_item) {
       print $curr_item['link'];
       ... etc ...
    }

    An item in the items array has the following valid keys:
      link             - URL to the current movie
      movie_id         - Netflix ID of the movie, can be used to get images
      title            - Name of the movie
      rating           - MPAA Rating
      genre            - Genre
      user_stars       - The star rating of the user
      predicted_stars  - The star rating from all Netflix users combined

   * History *

   0.1 - Original version for Expression Engine by Paul Burdick
   0.2 - Modifications by James McDuffie:
      * Removed expression engine specifics
      * Made class more generic
      * Added parsing of star ratings

   Plugin based off class written by Paul Burdick for ExpressionEngine
   reedmaniac@pmachine.com
   http://plugins.pmachine.com

  */

define(NETFLIX_PARSE_VERSION, '0.2'); 

Class NetFlixParse {

  // Period between cache refreshes (in minutes)
  var $cache_refresh = 1440;

  // Data from cache file
  var $cache_data    = '';

  // Path to cache file.
  var $cache_file    = '';
  
  // Path to cache file's time file.
  var $cache_tfile   = '';

  // URL for retrieving Queue
  var $netflix_url   = 'http://www.netflix.com/Queue';

  // ID for NetFlix account
  var $netflix_id    = '';

  // Items Data
  var $items	     = array();

  //-------------------------------------
  //  Constructor
  //-------------------------------------

  function NetFlixParse($id, $root_cache_path, $cache_refresh = 1440)
  {
   	
    // -------------------------------
    //  Set Parameters 
    // -------------------------------
    	
    $this->netflix_id = $id;

    if ($this->netflix_id == '') {
      return $this->return_data;
    }
    	
    // -------------------------------
    //  Check and Retrive Cache
    // -------------------------------

    $this->cache_refresh = $cache_refresh;    	
    $this->cache_file	= $root_cache_path . '/' . md5($this->netflix_id);
    $this->cache_tfile	= $this->cache_file . '_t';
        
    // -------------------------------
    // Retrieve NetFlix Data
    // -------------------------------
        
    $this->netflix_cache();
    	
    if ($this->cache_data == '') {
      return;
    }

  }
  // END
	
  //-------------------------------------
  //  Check for expired cache
  //-------------------------------------   

  function netflix_cache()
  {       

    // --------------------------
    // Check Cache
    // --------------------------
    if ( ! is_dir(dirname($this->cache_file))) {
      if ( ! @mkdir(dirname($this->cache_file), 0777)) {
	return false;
      }
    }

    if ( !file_exists($this->cache_file) ||  
	 !file_exists($this->cache_tfile)) {

      $this->retrieve_data();

    } else {
      $fp = @fopen($this->cache_tfile, 'r+b');
      $sp = @fopen($this->cache_file, 'r+b');
        	
      if ( ! is_resource($fp) ||  ! is_resource($sp)) {

	return false;

      } else {

	flock($fp, LOCK_SH);
	$timestamp = trim(@fread($fp, filesize($this->cache_tfile)));
	flock($fp, LOCK_UN);
	fclose($fp);
        
	if (time() > ($timestamp + ($this->cache_refresh * 60))) {
	  $this->retrieve_data();  
	} else {
	  flock($sp, LOCK_SH);
	  $this->cache_data = trim(@fread($sp, filesize($this->cache_file)));
	  flock($sp, LOCK_UN);
	  fclose($sp);
	}
      }
    }	

    // Remove line breaks and tabs
    $this->cache_data =& 
      preg_replace("/(\r\n)|(\r)|(\n)|(\t)/", ' ', $this->cache_data);
        
    return true;
        
  }
  // END

  //-------------------------------------
  //  Retreive NetFlix HTML File
  //-------------------------------------   

  function retrieve_data()
  {	
    $cookies = "Cookie: NetflixShopperId={$this->netflix_id};\r\n";
			
    $target = parse_url($this->netflix_url);
			
    $fp = @fsockopen($target['host'], 80, $errno, $errstr, 15);
			
    if (! is_resource($fp))
      {
	return false;
      }
			
    // Send the request
    fputs ($fp,"GET " . $this->netflix_url . " HTTP/1.0\r\n" ); 
    fputs ($fp,"Host: " . $target['host'] . "\r\n" ); 
    fputs ($fp, $cookies);
    fputs ($fp,"User-Agent: NetflixCheck/1.0\r\n\r\n");
			
    $getting_headers = true;
			
    while ( ! feof($fp))
      {
	$line = fgets($fp, 4096);
			
	if ($getting_headers == false)
	  {
	    $this->cache_data .= $line;
	  }
	elseif (trim($line) == '')
	  {
	    $getting_headers = false;
	  }
      }
	
    @fclose($fp); 
    	
    // -----------------------------------
    // Put into cache file
    // -----------------------------------
    		
    if ($this->cache_data != '')
      {	
	// Write Cache Time File
	if ($fp = @fopen($this->cache_tfile, 'wb'))
	  {
	    flock($fp, LOCK_SH);
	    fwrite($fp, time());
	    flock($fp, LOCK_UN);
	    fclose($fp);
	  }
    			
	// Write Cache File
	if ($fp = @fopen($this->cache_file, 'wb'))
	  {
	    flock($fp, LOCK_SH);
	    fwrite($fp, $this->cache_data);
	    flock($fp, LOCK_UN);
	    fclose($fp);
	  }
    			
	return true;
			
      }   
		
    return false;
  }
  // END

  // --------------------------------
  // --------------------------------

  function get_items() {
    return $this->items;
  }
	
  // --------------------------------
  //  Parse Data
  // --------------------------------

  function parse_data($which='queue')
  {
    $tag = ($which == 'queue') ? 'bd' : 'or';
    	
    if (preg_match_all("/\<tr class\=".$tag."\>(.+?)\<\/tr\>/", $this->cache_data, $matches))
      {
	for($i=0; $i < sizeof($matches['0']); $i++)
	  {
	    if (preg_match_all("/\<td.+?\>(.+?)\<\/td\>/", $matches['1'][$i], $matches2))
	      {
		for($t=1; $t < sizeof($matches2['1']); $t++)
		  {
		    switch($t)
		      {
		      case '1':
			// Link, title, and item_id									
			if (preg_match("/\<a href\=\"((.+?)movieid=(.+?))\"\>(.+?)\<\/a\>/", $matches2['1'][$t], $matches3))
			  {
			    $this->items[$i+1]['link']		=  $matches3['1'];
			    $this->items[$i+1]['movie_id'] 	=& $matches3['3'];
			    $this->items[$i+1]['title']		=  $matches3['4'];								
			  }
			else
			  {
			    continue;
			  }
			break;
		      case '2':
			// Rating/MPAA
			$this->items[$i+1]['rating'] = $matches2['1'][$t];
			break;
		      case '3':
			// Genre
			$this->items[$i+1]['genre'] = $matches2['1'][$t];
			break;
		      case '4':
			if (preg_match("/StarbarInsert\((.*)\)\;/", $matches2['1'][$t], $starbar_options) ) {
			  list( $itemId, $starType, $numStars,
				$predictedRating, $isNotInterested, 
				$isNoOpinion,
				$showNI, $showNiMini, $showClear, 
				$showNoOpinion,$refreshWhenChanged ) = split(",", $starbar_options['1']);
			  $this->items[$i+1]['user_stars'] = $numStars;
			  $this->items[$i+1]['predicted_stars'] = $predictedRating;
			}
			break;
		      }
		  }
	      }			
	  }			
      }
  }
  // END

} // END NetFlixParse class
?>